// lab2.c 
// Drake Vidkjer
// Created on 10-06-17
// Updated on 10-16-17

//  HARDWARE SETUP:
//  PORTA is connected to the segments of the LED display. and to the pushbuttons.
//  PORTA.0 corresponds to segment a, PORTA.1 corresponds to segement b, etc.
//  PORTB bits 4-6 go to a,b,c inputs of the 74HC138.
//  PORTB bit 7 goes to the PWM transistor base.

#define F_CPU 16000000 // cpu speed in hertz 
#define TRUE 1
#define FALSE 0
#include <avr/io.h>
#include <util/delay.h>

//Segment definitions, making it easier to set specific segments
/* Dont need these, dont need to select specific segments
#define SEG_A  0x01
#define SEG_B  0x02
#define SEG_C  0x04
#define SEG_D  0x08
#define SEG_E  0x10
#define SEG_F  0x20
#define SEG_G  0x40
#define SEG_DP 0x80
*/

//Digit select definitions, making it easier to select specific digits
#define DIG_1_SEL 0x10
#define DIG_2_SEL 0x20
#define DIG_3_SEL 0x40

//holds data to be sent to the segments. logic zero turns segment on
uint8_t segment_data[4];
//0 = true, 1 = false
uint8_t newNumb = 0;
uint8_t pinALast = 0x00;

//Small helper funcitons
//clear a segment by writing all 1's to port A.
void clearDig()
{
	PORTA = 0xFF;
}

//enable the tri state buffer with appropiate PORTB pins
void enableBuff()
{
	PORTB |= DIG_1_SEL | DIG_2_SEL | DIG_3_SEL;
}

//enable the buttons by pulling DDRA low
void enableBut()
{
	DDRA = 0x00;
	PORTA = 0xFF;
}

//enable LED's but pulling DDRA high
void enableLED()
{
	DDRA = 0xFF;
	//make sure tri state buffer is off
	PORTB |= DIG_3_SEL;
	PORTB = PORTB & ~(DIG_1_SEL | DIG_2_SEL);
}

//******************************************************************************
//                            chk_buttons                                      
//Checks the state of the buttons on PINA using a debounce method  
//returns the new count value once the button has been pressed for 12 cycles
//if button is not pressed for 12 cycles just returns the old count
//
uint16_t chk_buttons(uint16_t count) 
{
	uint16_t state = 0;
	uint8_t cycles = 0;
	
	//if pina is not entirely high that means that a button has been pressed, and if it doesnt equal
	//previous pina then that means theres been a change
	while(PINA != 0xFF && PINA != pinALast)
	{
		//if the same button is pressed, then increment cycles
		if(state == PINA)
		{
			cycles++;
		}
		//once button has been pressed 12 times, then return count + the new value
		if(cycles == 12)
		{
			//calculates the new value for count based on the old value and the value of PINA
			count = count + (0xFF - state);
			if(count > 1023)
			{
				count = count - 1023;
			}
			newNumb = 0;
			pinALast = PINA;
			return count;
		}
		state = PINA;
	}
	//store the last state of pina to compare later
	pinALast = PINA;
	return count;
}
//******************************************************************************

//***********************************************************************************
//                                   intToSeg                                  
//takes the 16-bit binary input values in the array and replaces them with the appropriate hex 
//BCD segment code in the array segment_data for display.                       
//array is loaded at exit as:  |digit3|digit2|colon|digit1|digit0|
void intToSeg()
{
	uint8_t i;
	for(i = 0; i < 4; i++)
	{
		switch(segment_data[i])
		{
	     case 0:
	       segment_data[i] = 0xC0;
	       break;
	     case 1:
	       segment_data[i] = 0xF9;
	       break;
	     case 2:
	       segment_data[i] = 0xA4;
	       break;
	     case 3:
	       segment_data[i] = 0xB0;
	       break;
	     case 4:
	       segment_data[i] = 0x99;
	       break;
	     case 5:
	       segment_data[i] = 0x92;
	       break;
	     case 6:
	       segment_data[i] = 0x82;
	       break;
	     case 7:
	       segment_data[i] = 0xF8;
	       break;
	     case 8:
	       segment_data[i] = 0x80;
	       break;
	     case 9:
	       segment_data[i] = 0x98;
	       break;
	     case 12:
		   segment_data[i] = 0xFF;
		   break;
		}
	}
}
//***********************************************************************************

//***********************************************************************************
//                                   setDig                                    
//sets the correct digit using the segment_data array 
//BCD segment code in the array segment_data for display.
void setDig(uint8_t i)
{
	if(i == 0)
	{
		PORTA = segment_data[i];
		PORTB |= DIG_3_SEL;
		PORTB = PORTB & ~(DIG_1_SEL | DIG_2_SEL);
	}
	else if(i == 1)
	{
		PORTA = segment_data[i];
		PORTB |= DIG_1_SEL | DIG_2_SEL;
		PORTB = PORTB & ~(DIG_3_SEL);
	}
	else if(i == 2)
	{
		PORTA = segment_data[i];
		PORTB |= DIG_1_SEL;
		PORTB = PORTB & ~(DIG_2_SEL | DIG_3_SEL);
	}
	else if(i == 3)
	{
		PORTA = segment_data[i];
		PORTB = PORTB & ~(DIG_1_SEL | DIG_2_SEL | DIG_3_SEL);
	}
}
//***********************************************************************************

//***********************************************************************************
//                                   dipslay                                    
//takes a 16-bit input value and displays this value on the 7 segment LED 
//BCD segment code in the array segment_data for display.                       
//array is loaded at exit as:  |digit3|digit2|colon|digit1|digit0|
void display(uint16_t count)
{
	uint8_t i;
	uint16_t j;
	
	//check if there is a new number, if not then dont do the conversion
	if(newNumb == 0)
	{
		//this is needed for the leading zeros since count is changed before it is needed to compare.
		j = count;
		
		//place digits into array
		segment_data[3] = count % 10;
		count = count / 10;
		segment_data[2] = count % 10;
		count = count / 10;
		segment_data[1] = count % 10;
		count = count / 10;
		segment_data[0] = count % 10;
		
		//this fixes the leading zero problem by replacing them with 12's which will be interpreted later
		if(j < 1000)
		{
			segment_data[0] = 12;
		}
		if(j < 100)
		{
			segment_data[1] = 12;
		}
		if(j < 10)
		{
			segment_data[2] = 12;
		}
		
		//convert the integers in the array into the appropriate hex values for the 7 seg
		//and let uc know the new number has been put in the array
		intToSeg();
		newNumb = 1;
	}
	
	//now actually display the data in the array on the display
	for(i = 0; i < 4; i++)
	{
		clearDig();
		//_delay_us(500);
		setDig(i);
		_delay_us(500);
	}
}
//***********************************************************************************

//***********************************************************************************
uint8_t main()
{
	//set port bits 4-7 B as outputs
	DDRB = 0xF0;
	
	//initialize local variables
	uint16_t count = 0;
	
	while(1)
	{
		//first set porta to output then display number in count
		enableLED();
		display(count);
		
		//switch to input and check for button presses
		enableBut();
		enableBuff();
		count = chk_buttons(count);
	}
	return 0;
}
